/*
 * Copyright (c) 2008 Jiri Svoboda
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * - The name of the author may not be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/** @addtogroup rtld rtld
 * @brief
 * @{
 */ 
/**
 * @file
 */

#include <sys/types.h>
#include <align.h>
#include <tls.h>

static void kputint(unsigned i)
{
	unsigned dummy;
	asm volatile (
		"movl $30, %%eax;"
		"int $0x30"
		: "=d" (dummy) /* output - %edx clobbered */
		: "d" (i) /* input */
		: "%eax","%ecx" /* all scratch registers clobbered */
	);	
}

typedef struct {
	unsigned long int ti_module;
	unsigned long int ti_offset;
} tls_index;

void __attribute__ ((__regparm__ (1)))
    *___tls_get_addr(tls_index *ti);

void __attribute__ ((__regparm__ (1)))
    *___tls_get_addr(tls_index *ti)
{
	size_t tls_size;
	uint8_t *tls;

	/* Calculate size of TLS block */
	tls_size = ALIGN_UP(&_tbss_end - &_tdata_start, &_tls_alignment);

	/* The TLS block is just before TCB */
	tls = (uint8_t *)__tcb_get() - tls_size;

	return tls + ti->ti_offset;
}

/** @}
 */
